// Connect.cpp : Implementation of CConnect
#include "stdafx.h"
#include "AddIn.h"
#include "Connect.h"
#include "DotOleErrors.h"
#include "CCommandBar0.h"
#include "CCommandBars.h"
#include "CCommandBarControl.h"
#include "CCommandBarControls.h"
#include "WordFind.h"
#include "CDocument0.h"

typedef enum {
	msoControlCustom = 0,
	msoControlButton = 1,
	msoControlEdit = 2,
	msoControlDropdown = 3,
	msoControlComboBox = 4,
	msoControlButtonDropdown = 5,
	msoControlSplitDropdown = 6,
	msoControlOCXDropdown = 7,
	msoControlGenericDropdown = 8,
	msoControlGraphicDropdown = 9,
	msoControlPopup = 10,
	msoControlGraphicPopup = 11,
	msoControlButtonPopup = 12,
	msoControlSplitButtonPopup = 13,
	msoControlSplitButtonMRUPopup = 14,
	msoControlLabel = 15,
	msoControlExpandingGrid = 16,
	msoControlSplitExpandingGrid = 17,
	msoControlGrid = 18,
	msoControlGauge = 19,
	msoControlGraphicCombo = 20,
	msoControlPane = 21,
	msoControlActiveX = 22,
	msoControlSpinner = 23,
	msoControlLabelEx = 24,
	msoControlWorkPane = 25,
	msoControlAutoCompleteCombo = 26
} MsoControlType;

typedef enum {
	msoButtonAutomatic = 0,
	msoButtonIcon = 1,
	msoButtonCaption = 2,
	msoButtonIconAndCaption = 3,
	msoButtonIconAndWrapCaption = 7,
	msoButtonIconAndCaptionBelow = 11,
	msoButtonWrapCaption = 14,
	msoButtonIconAndWrapCaptionBelow = 15
} MsoButtonStyle;


extern CAddInModule _AtlModule;

// When run, the Add-in wizard prepared the registry for the Add-in.
// At a later time, if the Add-in becomes unavailable for reasons such as:
//   1) You moved this project to a computer other than which is was originally created on.
//   2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
//   3) Registry corruption.
// you will need to re-register the Add-in by building the WordFindFacilitySetup project, 
// right click the project in the Solution Explorer, then choose install.


// CConnect
STDMETHODIMP CConnect::OnConnection(IDispatch *pApplication, AddInDesignerObjects::ext_ConnectMode /*ConnectMode*/, IDispatch *pAddInInst, SAFEARRAY ** /*custom*/ )
{
	pApplication->QueryInterface(__uuidof(IDispatch), (LPVOID*)&m_pApplication);
	pAddInInst->QueryInterface(__uuidof(IDispatch), (LPVOID*)&m_pAddInInstance);

	try
	{
		m_oWordApp.AttachDispatch( pApplication );
	}
	catch( DotOleException* e )
	{
		DotOleErrors::OleErrorMessages(e);
	}
	catch( DotOleDispatchException* e )
	{
		DotOleErrors::DispatchErrorMessages(e);
	}

	return S_OK;
}

STDMETHODIMP CConnect::OnDisconnection(AddInDesignerObjects::ext_DisconnectMode /*RemoveMode*/, SAFEARRAY ** /*custom*/ )
{
	m_pApplication = NULL;
	m_pAddInInstance = NULL;
	return S_OK;
}

STDMETHODIMP CConnect::OnAddInsUpdate (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}

STDMETHODIMP CConnect::OnStartupComplete (SAFEARRAY ** /*custom*/ )
{
	RemoveMenuItems();
	AddMenuItems();
	return S_OK;
}

STDMETHODIMP CConnect::OnBeginShutdown (SAFEARRAY ** /*custom*/ )
{
	RemoveMenuItems();
	return S_OK;
}

void CConnect::AddMenuItems()
{
	try
	{
		CCommandBars oCommBars = m_oWordApp.get_CommandBars();
		CCommandBar0 oMenuBar = oCommBars.get_ActiveMenuBar();
		CCommandBarControls oMenuBarControls = oMenuBar.get_Controls();

		long count = oMenuBarControls.get_Count();

		for (long iii=1; iii<=count; iii++)
		{
			CComVariant vLong(iii);
			CCommandBarControl oControl = oMenuBarControls.get_Item(vLong);
			if ( oControl.get_Caption().Compare( "&Window" ) == 0 )
			{
				MsoControlType type = msoControlPopup;
				m_oWordFindPopup = oMenuBarControls.Add( CComVariant(type), vOpt, 
					vOpt, CComVariant(iii), vOpt );
				m_oWordFindPopup.put_Tag( "WordFindFacility Popup" );
				m_oWordFindPopup.put_Caption( "&My Menu" );
				m_oWordFindPopup.put_Visible(TRUE);
				m_oWordFindPopup.put_TooltipText("A Fairly Useless Word Find Facility");
				break;
			}
		}
		
		CCommandBarControls controls = m_oWordFindPopup.get_Controls();
		MsoControlType type = msoControlButton;
		m_oWordFindButton = controls.Add( CComVariant(type), vOpt, vOpt, CComVariant(1), vOpt );
		m_oWordFindButton.put_Tag("WordFindFacility Popup");
		m_oWordFindButton.put_Caption("&Find...");
		m_oWordFindButton.put_Visible(TRUE);
		m_oWordFindButton.put_TooltipText("A Fairly Useless Word Find Facility");
		MsoButtonStyle style = msoButtonCaption;
		m_oWordFindButton.put_Style( style );
		m_oWordFindButton.put_OnAction( "WordFindFacility" );
	}

	catch( DotOleException* e )
	{
		DotOleErrors::OleErrorMessages(e);
	}
	catch( DotOleDispatchException* e )
	{
		DotOleErrors::DispatchErrorMessages(e);
	}
}

void CConnect::RemoveMenuItems()
{
	try
	{
		CCommandBars oCommBars = m_oWordApp.get_CommandBars();
		CCommandBar0 oMenuBar = oCommBars.get_ActiveMenuBar();
		CCommandBarControls oMenuBarControls = oMenuBar.get_Controls();

		long count = oMenuBarControls.get_Count();
		for( long iii=1; iii<=count; iii++ )
		{
			CComVariant vLong(iii);
			CCommandBarPopup oPopup = oMenuBarControls.get_Item(vLong);
			if ( oPopup.get_Caption().Compare( "&My Menu" ) == 0 )
			{
				CCommandBarControls oPopupControls = oPopup.get_Controls();
				while ( oPopupControls.get_Count() > 0 )
				{
					CCommandBarButton button = oPopupControls.get_Item( CComVariant((long)1) );
					button.Delete(vOpt);
				}
				oPopup.Delete(vOpt);
				count--;
			}
		}
	}
	catch( DotOleException* e )
	{
		DotOleErrors::OleErrorMessages(e);
	}
	catch( DotOleDispatchException* e )
	{
		DotOleErrors::DispatchErrorMessages(e);
	}
}
STDMETHODIMP CConnect::WordFind(void)
{
	// TODO: Add your implementation code here
	try
	{
		CDocument0 oActiveDocument = m_oWordApp.get_ActiveDocument();
		CWordFind dlg( &oActiveDocument );
		dlg.DoModal();
	}
	catch ( DotOleException* e )
	{
		DotOleErrors::OleErrorMessages(e);
	}
	catch ( DotOleDispatchException* e )
	{
		DotOleErrors::DispatchErrorMessages(e);
	}

	return S_OK;
}
